@@ -1,4 +1,4 @@
-/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
+/* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /*
  * Copyright (c) 2008-2010  litl, LLC
  *
@@ -75,32 +75,37 @@ gjs_is_registered_native_module(JSContext  *context,
 }
 
 /**
- * gjs_import_native_module:
- * @context:
- * @module_obj:
+ * gjs_load_native_module:
+ * @context: the #JSContext
+ * @parse_name: Name under which the module was registered with
+ *  gjs_register_native_module(), should be in the format as returned by
+ *  g_file_get_parse_name()
+ * @module_out: Return location for a #JSObject
  *
- * Return a native module that's been preloaded.
+ * Loads a builtin native-code module called @name into @module_out.
+ *
+ * Returns: true on success, false if an exception was thrown.
  */
 bool
-gjs_import_native_module(JSContext              *context,
-                         const char             *name,
-                         JS::MutableHandleObject module_out)
+gjs_load_native_module(JSContext              *context,
+                       const char             *parse_name,
+                       JS::MutableHandleObject module_out)
 {
     GjsDefineModuleFunc func;
 
     gjs_debug(GJS_DEBUG_NATIVE,
               "Defining native module '%s'",
-              name);
+              parse_name);
 
     if (modules != NULL)
-        func = (GjsDefineModuleFunc) g_hash_table_lookup(modules, name);
+        func = (GjsDefineModuleFunc) g_hash_table_lookup(modules, parse_name);
     else
         func = NULL;
 
     if (!func) {
         gjs_throw(context,
                   "No native module '%s' has registered itself",
-                  name);
+                  parse_name);
         return false;
     }
 
