@@ -34,7 +34,6 @@ G_BEGIN_DECLS
  * to the switch in log.c
  */
 typedef enum {
-    GJS_DEBUG_STRACE_TIMESTAMP,
     GJS_DEBUG_GI_USAGE,
     GJS_DEBUG_MEMORY,
     GJS_DEBUG_CONTEXT,
@@ -49,17 +48,9 @@ typedef enum {
     GJS_DEBUG_GBOXED,
     GJS_DEBUG_GENUM,
     GJS_DEBUG_GPARAM,
-    GJS_DEBUG_DATABASE,
-    GJS_DEBUG_RESULTSET,
-    GJS_DEBUG_WEAK_HASH,
-    GJS_DEBUG_MAINLOOP,
-    GJS_DEBUG_PROPS,
-    GJS_DEBUG_SCOPE,
-    GJS_DEBUG_HTTP,
-    GJS_DEBUG_BYTE_ARRAY,
     GJS_DEBUG_GERROR,
     GJS_DEBUG_GFUNDAMENTAL,
-    GJS_DEBUG_PROXY,
+    GJS_DEBUG_GINTERFACE,
 } GjsDebugTopic;
 
 /* These defines are because we have some pretty expensive and
